#!/bin/sh
# 
# 2008      Nico Schottelius (nico-cconf at schottelius.org)
# 
# This file is part of cconf.
#
# cconf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# cconf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with cconf. If not, see <http://www.gnu.org/licenses/>.
#
# Main script
#

hier="$(pwd -P)"        # where to search for cconf
dir="${hier}/cconf"     # the main configuration directory
exec="${dir}/libexec"   # contains executables used by cconf
tmpdir="${dir}/tmp"     # directory for temporary files
progdir="programs"      # subdir for program checks
libdir="libraries"      # subdir for libraries checks
searchfor="searchfor"   # which subdir contains things to search for

version="0.0.2"
version_name="17th of April 2008"
author="Nico Schottelius"
fullversion="cconf ${version} (${version_name}) written by ${author}"

self=${0##*/}
tmpfile=$(mktemp /tmp/${self}.XXXXXXXXXXXXXXXXXXXXX)

################################################################################
# functions: stolen from ccollect



# functions first
_echo()
{
   echo "${self}> $@"
}

_exit_err()
{
   _echo "$@"
   rm -f "$tmpfile"
   exit 1
}
################################################################################
# checks: argv, cconf-dir

if [ ! -d "$dir" ]; then
   _exit_err "No configuration directory ${dir} found"
fi


################################################################################
echo "${fullversion}"
echo ""

echo -n "Preparing tmp ... "
mkdir -p "${tmpdir}"             || exit 1
mkdir -p "${tmpdir}/${progdir}"  || exit 1
mkdir -p "${tmpdir}/${libdir}"   || exit 1
echo "ok"

echo "Detecting needed programs ..."

cd "${dir}/${searchfor}/${progdir}"
for prog in *; do
   echo -n "Checking for ${prog} ... "
   if [ -x "${exec}/${progdir}/${prog}" ]; then
      tmp="$("${exec}/${progdir}/${prog}")"
   else
      tmp="$(which "${prog}")"
   fi
   ret=$?

   if [ "$ret" -ne 0 ]; then
      echo "missing"
      exit 1
   else
      eval ${prog}=\"${tmp}\"
      eval echo "\$${prog}"
      file="${tmpdir}/${progdir}/${prog}"

      cat << eof > "${file}"
#!/bin/sh
#
# Autogenerated by ${fullversion}
# Created at $(date +%Y-%m-%d-%H:%M:%S)
#

eof
      eval echo "\$${prog} \\\"\\\$@\\\"" >> "${file}"
      chmod 0700 "${file}"
   fi
done


## Headers

## Libraries
echo "Detecting needed libraries ..."

if [ ! -x "${tmpdir}/${progdir}/cc" ]; then
   echo "No cc detected (add cc to ${searchfor}/${progdir})"
   echo "Skipping library detection"
else
   cfile="${tmpfile}.c"
   ofile="${tmpfile}.o"
   echo 'int main() { return 0; }' > "$cfile"

   cd "${dir}/${searchfor}/${libdir}"
   for lib in $(ls -1); do
      echo -n "Checking for lib${lib} ... "
      if [ -x "${exec}/${libdir}/${lib}" ]; then
         tmp="$("${exec}/${libdir}/${lib}")"
      else
         tmp="$("${tmpdir}/${progdir}/cc" -l"${lib}" "$cfile" -o "$ofile" 2>/dev/null)"
      fi
      ret=$?

      if [ "$ret" -ne 0 ]; then
         echo "missing"
         exit 1
      else
         echo "found"
      fi
   done
   rm "$cfile" "$ofile"
fi

echo "Finished."
